/*
	STM32F0DISCOVERY tutorial
	SysTick-based blinker with table-driven init
	gbm, 12'2012
*/

#include "stm32f0xx.h"
//========================================================================
// defs for STM32F05x chips
#define GPIO_MODER_OUT	1
//========================================================================
// defs for STM32F0DISCOVERY board
#define LED_PORT		GPIOC
#define BLUE_LED_BIT	8
#define GREEN_LED_BIT	9
//========================================================================
// Timings
#define SYSCLK_FREQ		HSI_VALUE
#define SYSTICK_FREQ	100	// 100 Hz -> 10 ms
#define BLINK_PERIOD	50	// * 10 ms
//========================================================================
struct init_entry_ {
	volatile uint32_t *loc;
	uint32_t value;
};

static __INLINE void writeregs(const struct init_entry_ *p)
{
	for (; p->loc; p ++)
		*p->loc = p->value;
}
//========================================================================
void SystemInit(void)
{
	FLASH->ACR = FLASH_ACR_PRFTBE;	// enable prefetch
}
//========================================================================
static const struct init_entry_ init_table[] = 
{
	// port setup
	{&RCC->AHBENR, RCC_AHBENR_GPIOCEN},	// GPIOC
	{&LED_PORT->MODER, GPIO_MODER_OUT << (GREEN_LED_BIT << 1)
		| GPIO_MODER_OUT << (BLUE_LED_BIT << 1)},	// set LED pins as outputs
	{(__IO uint32_t *)&LED_PORT->ODR, 1 << GREEN_LED_BIT},	// green initially ON
	//SysTick setup
	{&SysTick->LOAD, SYSCLK_FREQ / SYSTICK_FREQ - 1},
	{&SysTick->VAL, 0},
	{&SysTick->CTRL, SysTick_CTRL_CLKSOURCE_Msk | SysTick_CTRL_TICKINT_Msk
		| SysTick_CTRL_ENABLE_Msk},
	{&SCB->SCR, SCB_SCR_SLEEPONEXIT_Msk},	// sleep while not in handler
	{0, 0}
};
//========================================================================
int main(void)
{
	writeregs(init_table);
	__WFI();	// go to sleep
}
//========================================================================
void SysTick_Handler(void)
{
	static uint8_t blink_timer = BLINK_PERIOD;
	
	if (-- blink_timer == 0)
	{
		blink_timer = BLINK_PERIOD;
		// toggle both LEDs
		LED_PORT->ODR ^= 1 << GREEN_LED_BIT | 1 << BLUE_LED_BIT;
	}
}
