/*
	STM32F0DISCOVERY tutorial
	TIM3 RC Servo demo
	Servo output from PC9 - same as green LED
	gbm, 04'2013
*/

#include "stm32f0xx.h"
//========================================================================
// defs for STM32F05x chips
#define GPIO_MODER_OUT	1
#define GPIO_MODER_AF	2

#define TIM_CCMR2_OC3M_PWM1	0x0060	// OC3M[2:0] - PWM mode 1
#define TIM_CCMR2_OC4M_PWM1	0x6000	// OC3M[2:0] - PWM mode 1

typedef __IO uint32_t * __IO32p;
//========================================================================
// defs for STM32F0DISCOVERY board
#define LED_PORT		GPIOC
#define BLUE_LED_BIT	8
#define GREEN_LED_BIT	9

#define BUTTON_PORT		GPIOA
#define BUTTON_BIT		0

#define BLUE_LED_PWM	TIM3->CCR3
#define GREEN_LED_PWM	TIM3->CCR4
//========================================================================
// Timings
#define SYSCLK_FREQ	HSI_VALUE

// PWM constants
#define PWM_FREQ	50	// Hz
#define PWM_STEPS	2000	// 10 us step
#define	PWM_CLK		SYSCLK_FREQ
#define	PWM_PRE		(PWM_CLK / PWM_FREQ / PWM_STEPS)
#define SRV_MAX		(250 - 1)	// * 10 us
#define SRV_MIN		(50 - 1)	// * 10 us

#define LED_DIM	1
#define LED_MAX (PWM_STEPS - 1)

//========================================================================
struct init_entry_ {
	volatile uint32_t *loc;
	uint32_t value;
};

static __INLINE void writeregs(const struct init_entry_ *p)
{
	for (; p->loc; p ++)
		*p->loc = p->value;
}
//========================================================================
void SystemInit(void)
{
	FLASH->ACR = FLASH_ACR_PRFTBE;	// enable prefetch
}
//========================================================================
static const struct init_entry_ init_table[] = 
{
	// port setup
	{&RCC->AHBENR, RCC_AHBENR_GPIOCEN |  RCC_AHBENR_GPIOAEN},	// GPIOC, GPIOA
	{&LED_PORT->MODER, GPIO_MODER_AF << (GREEN_LED_BIT << 1)
		| GPIO_MODER_AF << (BLUE_LED_BIT << 1)},	// set LED pins as AF
	{&RCC->AHBENR, RCC_AHBENR_GPIOAEN},	// GPIOA
	// PWM timer setup - TIM3
	{&RCC->APB1ENR, RCC_APB1ENR_TIM3EN},	// TIM3
	{(__IO32p)&TIM3->PSC, PWM_PRE - 1},
	{(__IO32p)&TIM3->ARR, PWM_STEPS - 1},
	// blue - CH3, green - CH4
	{(__IO32p)&BLUE_LED_PWM, LED_DIM},
	{(__IO32p)&GREEN_LED_PWM, SRV_MIN},
	{(__IO32p)&TIM3->CCMR2, TIM_CCMR2_OC4M_PWM1 | TIM_CCMR2_OC4PE
		| TIM_CCMR2_OC3M_PWM1 | TIM_CCMR2_OC3PE},	// PWM mode 1, buffered preload
	{(__IO32p)&TIM3->CCER, TIM_CCER_CC4E | TIM_CCER_CC3E},	// enable CH3, 4 output
	{(__IO32p)&TIM3->DIER, TIM_DIER_UIE},	// enable update interrupt
	{(__IO32p)&TIM3->CR1, TIM_CR1_ARPE | TIM_CR1_CEN},	// auto reload, enable
	// interrupts and sleep
	{&NVIC->ISER[0], 1 << TIM3_IRQn},	// enable interrupt
	{&SCB->SCR, SCB_SCR_SLEEPONEXIT_Msk},	// sleep while not in handler
	{0, 0}
};
//========================================================================
int main(void)
{
	writeregs(init_table);
	__WFI();	// go to sleep
}
//========================================================================
void TIM3_IRQHandler(void)
{
	static uint16_t srv_target = SRV_MAX;
	static uint8_t bstate = 0;
	static uint8_t utimer;
	static _Bool move = 0;

	TIM3->SR = ~TIM_SR_UIF;	// clear interrupt flag

	if ((bstate = (bstate << 1 & 0x7) | (BUTTON_PORT->IDR >> BUTTON_BIT & 1)) == 1)
	{
		// button pressed - start/stop
		move ^= 1;
		BLUE_LED_PWM ^= LED_MAX ^ LED_DIM;
	}
	
	if (utimer)
	{
		// pause at max/min
		-- utimer;
	}
	else if (move)
	{
		uint16_t pwmval = GREEN_LED_PWM;
		
		if (pwmval != srv_target)
			GREEN_LED_PWM = (pwmval < srv_target) ? pwmval + 1 : pwmval - 1;
		else
		{
			srv_target ^= SRV_MAX ^ SRV_MIN;
			utimer = PWM_FREQ;	// pause for 1 s
		}
	}
}
